/* this function does the real work of extracting data from the 
 * NRCVAX .cd file and outputting the sir input file.
 * returns 0 if function exits ok, -1 if function detects an
 * error condition.
 * 
 * Written by:
 *	Paul D. Boyle
 *	Department of Chemistry
 *	Box 8204
 *	North Carolina State University
 *	Raleigh, NC 27695-8204
 *
 *	March 1999
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */
#include <stdlib.h>
#include <stdio.h>

#include "sirinp.h"
#ifndef NRCVAX_H
#include "nrcvax.h"
#define NRCVAX_H
#endif


int ProcessCDFile( const struct data_files *data )
{
    Record_1  rec1;
    Record_2  rec2;
    Record_6  rec6;
    Record_7  rec7;

    FILE *cd_file,
         *sir_file;

    char *spgp,
         *formula,
         *nrc_title;

    AtomTypeInfo *atom_info;

/* open the files */
    cd_file = fopen( data->cd, "rb" );
    if( !cd_file ) {
        perror( data->cd );
        return -1;
    }

    sir_file = fopen( data->sir, "w" );
    if( !sir_file ) {
        perror( data->sir );
        return -1;
    }

/* get relevant data from .cd file */
    ReadRecordFromNRCFile( CD_FILE, 1, &rec1, cd_file );
    ReadRecordFromNRCFile( CD_FILE, 2, &rec2, cd_file );
    ReadRecordFromNRCFile( CD_FILE, 6, &rec6, cd_file );
    ReadRecordFromNRCFile( CD_FILE, 7, &rec7, cd_file );

    nrc_title = GetJobname( &rec1 );
    if(!nrc_title ) {
       fputs( "**Error** GetJobname() returned NULL. Exiting.\n",
              stderr );
       return -1;
    }
  
    spgp = ExtractSpaceGroupSymbol( &rec2 );
    if(!spgp ) {
       fputs( "**Error** ExtractSpaceGroupSymbol()"
              " returned NULL. Exiting.\n",
              stderr );
       return -1;
    }

    if( rec6.ntyps > 5 )
       atom_info = GetAtomTypeInfo( &rec6, &rec7 );
    else
       atom_info = GetAtomTypeInfo( &rec6, NULL );

    if( !atom_info ) {
        fprintf( stderr,
         "**Error ** GetAtomTypeInfo() returned NULL."
         " Number of atom types %d\n", 
          rec6.ntyps );
        return -1;
    }

    formula = GetCellContents( atom_info, rec6.ntyps );
    if( !formula ) {
      fputs( "**Error** GetCellContents() returned NULL. Exiting.\n", stderr );
      return -1;
    }

/* Now print out the sir input file */
    fprintf( sir_file, SIR_HEADER_FORMAT,  data->job, nrc_title );
    fprintf( sir_file, SIR_CELL_FORMAT, rec1.direct.a, rec1.direct.b,
                       rec1.direct.c, rec1.direct.alpha, rec1.direct.beta,
                       rec1.direct.gamma );
    fprintf( sir_file, SIR_SPACE_FORMAT, spgp );
    fprintf( sir_file, SIR_CONT_FORMAT, formula );
    fprintf( sir_file, SIR_REFL_FORMAT, data->hkl );
    fprintf( sir_file, SIR_DEFAULT_INST );

    free( nrc_title );
    free( spgp );
    free( formula );
    fclose( cd_file );
    fclose( sir_file );
    return 0;
}
